# todo init should read json config file or env
from .models import File


class FileIndexer(object):
    def __init__(self, download_dir):
        from smb.SMBConnection import SMBConnection
        from socket import gethostname

        self.username = ""
        self.password = ""
        self.server_name = "nas-2d-46-c2"
        self.client_name = gethostname()
        self.share_name = "media"
        self.conn = SMBConnection(username=self.username, password=self.password,
                                  my_name=self.client_name, remote_name=self.server_name)
        self.conn.connect(ip="192.168.0.14", port=139)
        self.download_dir = download_dir
        self.full_file_list = []
        """ 
        full_file_list will be a list of dicts containing:
        file_name, file_ext, path, parent, size, dir=, normal=, ReadOnly=, etc...
        Basically, it's the values from the smb.base.SharedFile class + parent + file_ext
        We want to include parent to enable indexing by directory
        We want to include file_ext to enable quick tagging and icons
        """

    def index_filesystem(self, path, recurse=True):
        import re
        from datetime import datetime

        # List contents in root
        # Get Attributes of contents in root
        # If dir, add to list of dirs in root
        # Else, add to list of files in root
        # For each dir in root, list files
        dir_list = []
        ext_re_pattern = re.compile(r"(?<=[A-Za-z0-9\-{\}\[\]()\'=\"`+.]\.)[A-Za-z0-9]{1,4}$")

        files_obj = self.conn.listPath(self.share_name, path=path, search=65591, pattern="*", timeout=30)

        for f in files_obj:

            if f.filename != "." and f.filename != "..":
                if f.isDirectory:
                    dir_list.append(f.filename)

                file_ext = ext_re_pattern.search(f.filename)
                last_index_time = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
                last_index_time = datetime.fromisoformat(last_index_time)

                if file_ext:
                    file_ext = file_ext.group()

                file_dict = {
                            "filename": f.filename,
                            "parent_dir": path,
                            "full_path": path + f.filename,
                            "file_ext": file_ext,
                            "file_size": f.file_size,
                            "is_dir": f.isDirectory,
                            "last_index_time": last_index_time,
                            "create_time": datetime.fromtimestamp(f.create_time),
                            "last_access_time": datetime.fromtimestamp(f.last_access_time),
                            "last_write_time": datetime.fromtimestamp(f.last_write_time),
                            "last_attr_change_time": datetime.fromtimestamp(f.last_attr_change_time),
                            "alloc_size": f.alloc_size,
                            "file_attributes": f.file_attributes,
                            "is_normal": f.isNormal,
                            "is_read_only": f.isReadOnly
                             }

                self.full_file_list.append(file_dict)

        if recurse:
            for d in dir_list:
                self.index_filesystem(path + d + '/')

    def print_contents(self):
        for file in self.full_file_list:
            print("""
            filename              {0}
            parent_dir            {1}
            full_path             {2}  
            file_ext              {3}
            file_size             {4}
            is_dir                {5}
            last_index_time       {6}
            create_time           {7}
            last_access_time      {8}
            last_write_time       {9}
            last_attr_change_time {10}
            alloc_size            {11}
            file_attributes       {12}
            is_normal             {13}
            is_read_only          {14}
        
        
            """.format(
                        file["filename"],
                        file["parent_dir"],
                        file["full_path"],
                        file["file_ext"],
                        file["file_size"],
                        file["is_dir"],
                        file["last_index_time"],
                        file["create_time"],
                        file["last_access_time"],
                        file["last_write_time"],
                        file["last_attr_change_time"],
                        file["alloc_size"],
                        file["file_attributes"],
                        file["is_normal"],
                        file["is_read_only"]))
        print("There are {0} files in {1}".format(self.full_file_list.__len__(), self.share_name))

    def download_file(self, full_path, file_name):
        download_dir = r"{0}{1}".format(self.download_dir, file_name)
        with open(download_dir, "wb") as download:
            self.conn.retrieveFile(self.share_name, full_path, download)

    def upload_file(self, source_path, destination_path, file_name, db):
        full_path = destination_path + file_name
        with open(source_path, "rb") as file:
            self.conn.storeFile(self.share_name, full_path, file)
        self.index_filesystem(destination_path, recurse=False)
        self.commit_file_list(db)

    def commit_file_list(self, db):
        for entry in self.full_file_list:
            new_file = File(
                filename=entry["filename"],
                parent_dir=entry["parent_dir"],
                full_path=entry["full_path"],
                file_ext=entry["file_ext"],
                file_size=entry["file_size"],
                is_dir=entry["is_dir"],
                last_index_time=entry["last_index_time"],
                create_time=entry["create_time"],
                last_access_time=entry["last_access_time"],
                last_write_time=entry["last_write_time"],
                last_attr_change_time=entry["last_attr_change_time"],
                alloc_size=entry["alloc_size"],
                file_attributes=entry["file_attributes"],
                is_normal=entry["is_normal"],
                is_read_only=entry["is_read_only"]
            )
            db.session.add(new_file)
            db.session.commit()
